<?php

/**
 *
 * Copyright  2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Newsletter\Controller\Subscriber;

use Magento\Customer\Model\Account\Redirect as AccountRedirect;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\App\Action\Context;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Customer\Api\AccountManagementInterface;
use Magento\Customer\Helper\Address;
use Magento\Framework\UrlFactory;
use Magento\Customer\Model\Metadata\FormFactory;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Customer\Api\Data\RegionInterfaceFactory;
use Magento\Customer\Api\Data\AddressInterfaceFactory;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Customer\Model\Registration;
use Magento\Framework\Escaper;
use Magento\Customer\Model\CustomerExtractor;
use Magento\Framework\Exception\StateException;
use Magento\Framework\Exception\InputException;

class NewLandingRegisterAction extends \Magento\Customer\Controller\Account\CreatePost {

    protected $jsonResultFactory;
    public $_helper;
    protected $_modelPreConfirm;
    protected $_modelThankYou;

     /**
     * @var AccountRedirect
     */
    protected $accountRedirect;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory
     */
    protected $cookieMetadataFactory;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\PhpCookieManager
     */
    protected $cookieMetadataManager;
    
    /**
     * Initialize dependencies.
     *
     * @param Context $context
     * @param SubscriberFactory $subscriberFactory
     * @param Session $customerSession
     * @param StoreManagerInterface $storeManager
     * @param CustomerUrl $customerUrl
     * @param CustomerAccountManagement $customerAccountManagement
     */
    public function __construct(
    Context $context, Session $customerSession, ScopeConfigInterface $scopeConfig, StoreManagerInterface $storeManager, AccountManagementInterface $accountManagement, Address $addressHelper, UrlFactory $urlFactory, FormFactory $formFactory, SubscriberFactory $subscriberFactory, RegionInterfaceFactory $regionDataFactory, AddressInterfaceFactory $addressDataFactory, CustomerInterfaceFactory $customerDataFactory, CustomerUrl $customerUrl, Registration $registration, Escaper $escaper, CustomerExtractor $customerExtractor, DataObjectHelper $dataObjectHelper, AccountRedirect $accountRedirect, \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory, \FiloBlu\Newsletter\Helper\Data $helper, \FiloBlu\Newsletter\Model\PreConfirm $modelPreConfirm, \FiloBlu\Newsletter\Model\ThankYou $modelThankYou
    ) {

        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_modelPreConfirm = $modelPreConfirm;
        $this->_modelThankYou = $modelThankYou;
        $this->accountRedirect = $accountRedirect;

        parent::__construct(
                $context, $customerSession, $scopeConfig, $storeManager, $accountManagement, $addressHelper, $urlFactory, $formFactory, $subscriberFactory, $regionDataFactory, $addressDataFactory, $customerDataFactory, $customerUrl, $registration, $escaper, $customerExtractor, $dataObjectHelper, $accountRedirect
        );
    }

    /**
     * New subscription action
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return void
     */
    public function execute() {

        if ($this->_helper->getConfig("general/enabled") && $this->getRequest()->isPost()) {

            $resultJson = $this->resultJsonFactory->create();

            $this->session->regenerateId();

            try {

                if (!$this->registration->isAllowed()) {
                    throw new \Exception(__('Registration is not allowed.'));
                }
                $address = $this->extractAddress();
                $addresses = $address === null ? [] : [$address];

                $customer = $this->customerExtractor->extract('customer_account_create', $this->_request);
                $customer->setAddresses($addresses);

                $password = $this->getRequest()->getParam('password');
                $confirmation = $this->getRequest()->getParam('password_confirmation');
                $redirectUrl = $this->session->getBeforeAuthUrl();

                $this->checkPasswordConfirmation($password, $confirmation);

                $customer = $this->accountManagement
                        ->createAccount($customer, $password, $redirectUrl);

                if ($this->getRequest()->getParam('is_subscribed', false)) {
                    $this->subscriberFactory->create()->subscribeCustomerById($customer->getId());
                }

                $this->_eventManager->dispatch(
                        'customer_register_success', ['account_controller' => $this, 'customer' => $customer]
                );

                $confirmationStatus = $this->accountManagement->getConfirmationStatus($customer->getId());
                if ($confirmationStatus === AccountManagementInterface::ACCOUNT_CONFIRMATION_REQUIRED) {

                    if ($this->getRequest()->getPost('preconfirm_id') && $this->getRequest()->getPost('preconfirm_id') != 0) {
                        $this->_helper->setPreConfirmCookie($this->getRequest()->getPost('preconfirm_id'));
                        $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->storeManager->getStore()->getBaseUrl() . $this->_modelPreConfirm->getUrlPreConfirmPageById($this->getRequest()->getPost('preconfirm_id'))->getData()["url_key"]]);
                    } else {
                        $email = $this->customerUrl->getEmailConfirmationUrl($customer->getEmail());

                        $this->messageManager->addSuccess(
                                __(
                                        'You must confirm your account. Please check your email for the confirmation link or <a href="%1">click here</a> for a new link.', $email
                                )
                        );
                        $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->storeManager->getStore()->getBaseUrl()]);
                    }
                } else {
                    $this->session->setCustomerDataAsLoggedIn($customer);

                    if ($this->getRequest()->getPost('thankyou_id') && $this->getRequest()->getPost('thankyou_id') != 0) {
                        $this->_helper->setThankYouCookie($this->getRequest()->getPost('thankyou_id'));
                        $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->storeManager->getStore()->getBaseUrl() . $this->_modelThankYou->getUrlThankYouPageById($this->getRequest()->getPost('thankyou_id'))->getData()["url_key"]]);
                    } else {
                        $this->messageManager->addSuccess($this->getSuccessMessage());
                        $requestedRedirect = $this->accountRedirect->getRedirectCookie();
                        if (!$this->scopeConfig->getValue('customer/startup/redirect_dashboard') && $requestedRedirect) {
                            $this->accountRedirect->clearRedirectCookie();
                            $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->_redirect->success($requestedRedirect)]);
                        } else {
                            $resultJson->setData(['success' => true, 'error' => false, 'redirect' => $this->storeManager->getStore()->getBaseUrl()]);
                        }
                    }
                }

                if ($this->getCookieManager()->getCookie('mage-cache-sessid')) {
                    $metadata = $this->getCookieMetadataFactory()->createCookieMetadata();
                    $metadata->setPath('/');
                    $this->getCookieManager()->deleteCookie('mage-cache-sessid', $metadata);
                }
            } catch (StateException $e) {
                $url = $this->urlModel->getUrl('customer/account/forgotpassword');
                // @codingStandardsIgnoreStart
                $message = __(
                        'There is already an account with this email address. If you are sure that it is your email address, <a href="%1">click here</a> to get your password and access your account.', $url
                );
                // @codingStandardsIgnoreEnd

                $resultJson->setData(['success' => false, 'error' => true, 'message' => $message]);
            } catch (InputException $e) {
                $messages = __($e->getMessage());

                foreach ($e->getErrors() as $error) {
                    $messages .= "<br>" . __($error->getMessage());
                }
                $resultJson->setData(['success' => false, 'error' => true, 'message' => $messages]);
            } catch (LocalizedException $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
            } catch (\Exception $e) {
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
            }

            return $resultJson;
        } else {
            return null;
        }
    }

    /**
     * Retrieve cookie manager
     *
     * @deprecated
     * @return \Magento\Framework\Stdlib\Cookie\PhpCookieManager
     */
    private function getCookieManager() {
        if (!$this->cookieMetadataManager) {
            $this->cookieMetadataManager = \Magento\Framework\App\ObjectManager::getInstance()->get(
                    \Magento\Framework\Stdlib\Cookie\PhpCookieManager::class
            );
        }
        return $this->cookieMetadataManager;
    }

    /**
     * Retrieve cookie metadata factory
     *
     * @deprecated
     * @return \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory
     */
    private function getCookieMetadataFactory() {
        if (!$this->cookieMetadataFactory) {
            $this->cookieMetadataFactory = \Magento\Framework\App\ObjectManager::getInstance()->get(
                    \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory::class
            );
        }
        return $this->cookieMetadataFactory;
    }

}
